#include "metropolishistogrambuilder.h"
#include "function.h"
#include "randomnumbergenerator.h"
#include <stdlib.h>
#include <iostream>
#include <cmath>

using namespace meanwalker;

// WARNING: This is an insanely BAD random number generator. For real
//          applications, use a decent one (e.g. the Mersenne twister,
//          available in the GNU Scientific Library)

class MyRandomNumberGenerator : public RandomNumberGenerator
{
public:
	MyRandomNumberGenerator()	{ }
	~MyRandomNumberGenerator()	{ }

	double pickRandomNumber()
	{
		double randomNumber = (1.0*((double)rand()/(RAND_MAX + 1.0)));

		return randomNumber;
	}
};

// This is the logarithm of the probability density we're going to
// sample: a gaussian distribution with specific mean and standard
// deviation

class LogGaussianFunction : public Function
{
public:
	LogGaussianFunction(double center, double sigma)
	{
		m_center = center;
		m_sigma = sigma;
	}

	size_t getDimension() const
	{
		return 1;
	}

	double operator()(const double *pCoords) const
	{
		double x = pCoords[0];
		double diff = (x-m_center)/m_sigma;
		return -diff*diff/2.0;
	}

	bool isLog() const
	{
		return true;
	}
private:
	double m_center, m_sigma;
};

// Our class derived from MetropolisHistogramBuilder reimplements the
// getHistogramMinMax method. This is the function that's used to determine
// the histogram boundaries when the values are recorded at first, and
// processed into a histogram later.

class MyBuilder : public MetropolisHistogramBuilder
{
public:
	MyBuilder(RandomNumberGenerator *pRng) : MetropolisHistogramBuilder(pRng)
	{
	}
protected:
	void getHistogramMinMax(size_t idx, const double *pValues, size_t numValues, double &minValue, double &maxValue)
	{
		double mean = 0;

		for (size_t s = 0 ; s < numValues ; s++)
			mean += pValues[s];

		mean /= (double)numValues;

		double stdDev = 0;

		for (size_t s = 0 ; s < numValues ; s++)
		{
			double diff = pValues[s] - mean;

			stdDev += diff*diff;
		}

		stdDev /= (double)numValues;
		stdDev = std::sqrt(stdDev);

		minValue = mean - 2.0*stdDev;
		maxValue = mean + 2.0*stdDev;
	}
};

int main(void)
{
	MyRandomNumberGenerator rndGen;
	MyBuilder builder(&rndGen);
	LogGaussianFunction f(10, 3);

	double minc[] = { -20 };
	double maxc[] = { 30 };
	double startCoord[] = { -20 };
	size_t numBins = 100;

	// This example is similar to the previous one, except now the Metropolis-Hastings
	// method is used for sampling instead of the Goodman-Weare one. In this case we
	// still need to set minimum and maximum coordinates to fix the step size. We
	// also let the walker start a bit more near the center.

	if (!builder.init(&f, minc, maxc, startCoord, numBins, false, true))
	{
		std::cerr << builder.getErrorString() << std::endl;
		return -1;
	}

	builder.run(10000, 1000000);
	builder.printHistograms();

	return 0;
}
