/*

  This file is a part of MeanWalker, a library which provides utilities to
  sample from probability distributions using methods like the 
  Metropilis-Hastings algorithm and Goodman-Weare algorithm.

  Copyright (C) 2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file goodmanwearehistogrambuilder.h
 */

#ifndef MEANWALKER_GOODMANWEAREHISTOGRAMBUILDER_H

#define MEANWALKER_GOODMANWEAREHISTOGRAMBUILDER_H

#include "meanwalkerconfig.h"
#include "histogrambuildertemplate.h"
#include "goodmanwearewalkers.h"
#include "histogram.h"

namespace meanwalker
{

/** This class uses the Goodman-Weare method to take samples from a probability
 *  distribution and automatically creates histograms for the marginalized
 *  distributions corresponding to each of the coordinates.
 *  This class uses the Goodman-Weare method to take samples from a probability
 *  distribution and automatically creates histograms for the marginalized
 *  distributions corresponding to each of the coordinates. It uses a GoodmanWeareWalkers
 *  instance to take the samples.
 */
class MEANWALKER_IMPORTEXPORT GoodmanWeareHistogramBuilder : private HistogramBuilderTemplate<GoodmanWeareWalkers>
{
public:
	/** The constructor of the class, in which the random number generator to be used
	 *  must be specified. */
	GoodmanWeareHistogramBuilder(RandomNumberGenerator *pRng);
	~GoodmanWeareHistogramBuilder();

	/** Initialize the builder.
	 *  Initialize the builder.
	 *  \param pProbFunction The probability density to be sampled.
	 *  \param pStartCoordCenter If the number of dimensions is larger than one, the starting coordinates
	 *                           of the walkers will be chosen randomly on a hypersphere with this center.
	 *                           In the dimension is equal to one, the walkers will be placed randomly
	 *                           in the interval [center-radius, center+radius].
	 *  \param startCoordRadius Radius of the hypersphere, or size of the interval (see previous parameter).
	 *  \param numBins Number of bins the histograms will have.
	 *  \param pHistMinCoords If not null, this specifies the minimum coordinates of the histograms. This
	 *                        does not impose a minimum coordinate a walker can have. If null, the width of the
	 *                        histograms will be determined automatically.
	 *  \param pHistMaxCoords If not null, this specifies the maximum coordinates of the histograms. Again,
	 *                        this does not impose a maximum coordinate a walker can have. If null, the width of
	 *                        the histograms will be determined automatically.
	 *  \param a Scale factor for the stretch move from the Goodman-Weare algorithm.
	 *  \param Nwalkers The number of walkers which will sample the probability distribution. Must be at
	 *                  least one more than the dimension, which is also the default value (specified when
	 *                  setting this parameter to 0).
	 */
	bool init(Function *pProbFunction, const double *pStartCoordCenter, double startCoordRadius,
		  size_t numBins, const double *pHistMinCoords = 0, const double *pHistMaxCoords = 0, 
		  double a = 2.0, int Nwalkers = 0);
	
	/** Initialize the builder.
	 *  Initialize the builder.
	 *  \param pProbFunction The probability density to be sampled.
	 *  \param pStartCoordCenter If the number of dimensions is larger than one, the starting coordinates
	 *                           of the walkers will be chosen randomly on a hypersphere, or rather a
	 *                           hyperellipsoid, with this center.
	 *                           In the dimension is equal to one, the walkers will be placed randomly
	 *                           in the interval [center-radius, center+radius].
	 *  \param pStartCoordRadii Radii of of the hyperellipsoid along each axis, or size of the 
	 *                          interval (see previous parameter).
	 *  \param numBins Number of bins the histograms will have.
	 *  \param pHistMinCoords If not null, this specifies the minimum coordinates of the histograms. This
	 *                        does not impose a minimum coordinate a walker can have. If null, the width of the
	 *                        histograms will be determined automatically.
	 *  \param pHistMaxCoords If not null, this specifies the maximum coordinates of the histograms. Again,
	 *                        this does not impose a maximum coordinate a walker can have. If null, the width of
	 *                        the histograms will be determined automatically.
	 *  \param a Scale factor for the stretch move from the Goodman-Weare algorithm.
	 *  \param Nwalkers The number of walkers which will sample the probability distribution. Must be at
	 *                  least one more than the dimension, which is also the default value (specified when
	 *                  setting this parameter to 0).
	 */
	bool init(Function *pProbFunction, const double *pStartCoordCenter, const double *pStartCoordRadii,
		  size_t numBins, const double *pHistMinCoords = 0, const double *pHistMaxCoords = 0, 
		  double a = 2.0, int Nwalkers = 0);
	
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::run;
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::clearHistograms;
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::getHistogram;
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::getHistograms;
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::printHistograms;
	using GoodmanWeareWalkers::getDimension;
	using GoodmanWeareWalkers::getBestCoordinates;
	using GoodmanWeareWalkers::getNumberOfWalkers;

	using errut::ErrorBase::getErrorString;
protected:
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::onNewSample;
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::getHistogramMinMax;
	using HistogramBuilderTemplate<GoodmanWeareWalkers>::onRecordedValuesProcessed;
};

#endif // MEANWALKER_GOODMANWEAREHISTOGRAMBUILDER_H

} // end namespace   

