/*

  This file is a part of MeanWalker, a library which provides utilities to
  sample from probability distributions using methods like the 
  Metropilis-Hastings algorithm and Goodman-Weare algorithm.

  Copyright (C) 2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file goodmanwearewalkers.h
 */

#ifndef MEANWALKER_GOODMANWEAREWALKERS_H

#define MEANWALKER_GOODMANWEAREWALKERS_H

#include "meanwalkerconfig.h"
#include "walker.h"
#include <errut/errorbase.h>
#include <vector>

namespace meanwalker
{

class RandomNumberGenerator;

/** This class implements the Goodman-Weare method for taking samples from a particular
 *  probability distribution.
 *  This class implements the Goodman-Weare method for taking samples from a particular
 *  probability distribution. Is does this by creating a number of 'walkers', items which
 *  perform a sort of biased random walk, which together sample the specified distribution.
 *  The probability distribution can be specified either by itself, or as the logarithm
 *  of the distribution. In the first case, multiplication by a constant factor doens't
 *  matter (i.e. it doesn't have to be normed), which in the second case means that adding
 *  a specific constant does not matter.
 *
 *  \b References
 *    \li <A href="http://msp.berkeley.edu/camcos/2010/5-1/p04.xhtml">J. Goodman and J. Weare, Ensemble samplers with affine invariance, Communications in Applied Mathematics and Computational Science, Vol 5 (2010), No. 1, p65-80</A>
 */
class MEANWALKER_IMPORTEXPORT GoodmanWeareWalkers : public Walker
{
public:
	/** Constructor, in which you need to specify the random number generator to be used. */
	GoodmanWeareWalkers(RandomNumberGenerator *pRndGen);
	~GoodmanWeareWalkers();

	/** Initialize the sampler.
	 *  Initialize the sampler.
	 *  \param pProbFunction The probability density to be sampled.
	 *  \param isLog Flag indicating whether the specified function returns a value proportional to the
	 *               actual probability, or a value equal to the logarithm of the probability (plus some
	 *               arbitrary constant).
	 *  \param pStartCoordCenter If the number of dimensions is larger than one, the starting coordinates
	 *                           of the walkers will be chosen randomly on a hypersphere with this center.
	 *                           In the dimension is equal to one, the walkers will be placed randomly
	 *                           in the interval [center-radius, center+radius].
	 *  \param startCoordRadius Radius of the hypersphere, or size of the interval (see previous parameter).
	 *  \param a Scale factor for the stretch move from the Goodman-Weare algorithm.
	 *  \param Nwalkers The number of walkers which will sample the probability distribution. Must be at
	 *                  least one more than the dimension, which is also the default value (specified when
	 *                  setting this parameter to 0).
	 */
	bool init(Function *pProbFunction, const double *pStartCoordCenter, double startCoordRadius,
		  double a = 2.0, int Nwalkers = 0);

	/** Initialize the sampler.
	 *  Initialize the sampler.
	 *  \param pProbFunction The probability density to be sampled.
	 *  \param isLog Flag indicating whether the specified function returns a value proportional to the
	 *               actual probability, or a value equal to the logarithm of the probability (plus some
	 *               arbitrary constant).
	 *  \param pStartCoordCenter If the number of dimensions is larger than one, the starting coordinates
	 *                           of the walkers will be chosen randomly on a hypersphere, or rather a
	 *                           hyperellipsoid, with this center.
	 *                           In the dimension is equal to one, the walkers will be placed randomly
	 *                           in the interval [center-radius, center+radius].
	 *  \param pStartCoordRadii Radii of of the hyperellipsoid along each axis, or size of the 
	 *                          interval (see previous parameter).
	 *  \param a Scale factor for the stretch move from the Goodman-Weare algorithm.
	 *  \param Nwalkers The number of walkers which will sample the probability distribution. Must be at
	 *                  least one more than the dimension, which is also the default value (specified when
	 *                  setting this parameter to 0).
	 */
	bool init(Function *pProbFunction, const double *pStartCoordCenter, const double *pStartCoordRadii,
		  double a = 2.0, int Nwalkers = 0);

	void walk(size_t numSteps);
	const double *getBestCoordinates() const							{ return &(m_bestCoords[0]); }

	/** Returns the number of 'walkers' used. */
	size_t getNumberOfWalkers() const								{ return m_walkers.size(); }
private:
	struct InternalWalker
	{
		std::vector<double> m_coords;
		double m_prob;
	};

	std::vector<double> m_bestCoords;
	std::vector<double> m_tmp;
	std::vector<InternalWalker> m_walkers;
	double m_a, m_aScale, m_aOffset;
	size_t m_nextStartPos;
};

} // end namespace   

#endif // MEANWALKER_GOODMANWEAREWALKERS_H

