/*

  This file is a part of MeanWalker, a library which provides utilities to
  sample from probability distributions using methods like the 
  Metropilis-Hastings algorithm and Goodman-Weare algorithm.

  Copyright (C) 2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "histogram2d.h"
#include <iostream>

namespace meanwalker
{

Histogram2D::Histogram2D()
{
	m_maxValueX = 0;
	m_minValueX = 0;
	m_maxValueY = 0;
	m_minValueY = 0;
	m_totalCount = 0;
	m_outsideCount = 0;
	m_binWidth = 1;
	m_binHeight = 1;
	m_numX = 0;
	m_numY = 0;
	m_doubleBinsX = 0;
	m_doubleBinsY = 0;
	m_binArea = 1;
}

Histogram2D::~Histogram2D()
{
}

bool Histogram2D::init(double minValueX, double maxValueX, size_t numBinsX,
		       double minValueY, double maxValueY, size_t numBinsY)
{
	if (m_histogram.size() != 0)
	{
		setErrorString("Already initialized");
		return false;
	}

	if (numBinsX < 1 || numBinsY < 1)
	{
		setErrorString("There should be at least one bin");
		return false;
	}

	m_histogram.resize(numBinsX*numBinsY);
	for (size_t i = 0 ; i < m_histogram.size() ; i++)
		m_histogram[i] = 0;

	m_minValueX = minValueX;
	m_maxValueX = maxValueX;
	m_minValueY = minValueY;
	m_maxValueY = maxValueY;

	m_binWidth = (maxValueX-minValueX)/(double)numBinsX;
	m_binHeight = (maxValueY-minValueY)/(double)numBinsY;

	m_doubleBinsX = (double)numBinsX;
	m_doubleBinsY = (double)numBinsY;

	m_totalCount = 0;
	m_outsideCount = 0;

	m_numX = numBinsX;
	m_numY = numBinsY;

	m_binArea = m_binWidth*m_binHeight;

	return true;
}

void Histogram2D::clear()
{
	size_t numBins = m_histogram.size();

	for (size_t i = 0 ; i < numBins ; i++)
		m_histogram[i] = 0;

	m_totalCount = 0;
	m_outsideCount = 0;
}

void Histogram2D::destroy()
{
	m_histogram.resize(0);

	m_maxValueX = 0;
	m_minValueX = 0;
	m_maxValueY = 0;
	m_minValueY = 0;
	m_totalCount = 0;
	m_outsideCount = 0;
	m_binWidth = 1;
	m_binHeight = 1;
	m_numX = 0;
	m_numY = 0;
	m_doubleBinsX = 0;
	m_doubleBinsY = 0;
	m_binArea = 1;
}

void Histogram2D::print(bool blocks) const
{
	if (m_histogram.size() == 0)
		return;

	if (blocks)
	{
		for (size_t y = 0 ; y < m_numY ; y++)
		{
			for (size_t x = 0 ; x < m_numX ; x++)
			{
				std::cout << (getBinCenterX(x) - m_binWidth/2.0) << "\t" 
					  << (getBinCenterY(y) - m_binHeight/2.0) << "\t" 
					  << getProbabilityDensity(x, y) << std::endl;
				std::cout << (getBinCenterX(x) + m_binWidth/2.0) << "\t" 
					  << (getBinCenterY(y) - m_binHeight/2.0) << "\t" 
					  << getProbabilityDensity(x, y) << std::endl;
			}

			std::cout << std::endl;

			for (size_t x = 0 ; x < m_numX ; x++)
			{
				std::cout << (getBinCenterX(x) - m_binWidth/2.0) << "\t" 
					  << (getBinCenterY(y) + m_binHeight/2.0) << "\t" 
					  << getProbabilityDensity(x, y) << std::endl;
				std::cout << (getBinCenterX(x) + m_binWidth/2.0) << "\t" 
					  << (getBinCenterY(y) + m_binHeight/2.0) << "\t" 
					  << getProbabilityDensity(x, y) << std::endl;
			}

			std::cout << std::endl;
		}
	}
	else
	{
		for (size_t y = 0 ; y < m_numY ; y++)
		{
			for (size_t x = 0 ; x < m_numX ; x++)
			{
				std::cout << (getBinCenterX(x)) << "\t" 
					  << (getBinCenterY(y)) << "\t" 
					  << getProbabilityDensity(x, y) << std::endl;
			}

			std::cout << std::endl;
		}
	}

	std::cout << std::endl;
	std::cout << std::endl;
}

} // end namespace   

