/*

  This file is a part of MeanWalker, a library which provides utilities to
  sample from probability distributions using methods like the 
  Metropilis-Hastings algorithm and Goodman-Weare algorithm.

  Copyright (C) 2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file metropoliswalker.h
 */

#ifndef MEANWALKER_METROPOLISWALKER_H

#define MEANWALKER_METROPOLISWALKER_H

#include "meanwalkerconfig.h"
#include "walker.h"
#include <errut/errorbase.h>
#include <vector>

namespace meanwalker
{

class RandomNumberGenerator;

/** This class implements the Metropolis-Hastings method for taking samples from a particular
 *  probability distribution.
 *  This class implements the Metropolis-Hastings method for taking samples from a particular
 *  probability distribution. Is does this by creating a 'walker', an item which
 *  performs a sort of biased random walk, so that it takes samples from the specified distribution.
 *  The probability distribution can be specified either by itself, or as the logarithm
 *  of the distribution. In the first case, multiplication by a constant factor doens't
 *  matter (i.e. it doesn't have to be normed), which in the second case means that adding
 *  a specific constant does not matter.
 */
class MEANWALKER_IMPORTEXPORT MetropolisWalker : public Walker
{
public:
	/** Constructor, in which you need to specify the random number generator to be used. */
	MetropolisWalker(RandomNumberGenerator *pRndGen);
	~MetropolisWalker();

	/** Initialize the sampler.
	 *  Initialize the sampler.
	 *  \param pProbFunction The probability density to be sampled.
	 *  \param pMinCoords Sets the minimum coordinates which are used together with the maximum coordinates
	 *                    to calculate the scale of the steps used in the random walk.
	 *  \param pMaxCoords Sets the maximum coordinates which are used together with the minimum coordinates
	 *                    to calculate the scale of the steps used in the random walk.
	 *  \param pStartCoords If not null, the walker will start at these coordinates. If null, a random position
	 *                    between \c pMinCoords and \c pMaxCoords is used as starting point.
	 *  \param keepInside If true, the random walk is not allowed to go beyond the boundaries specified by
	 *                    the minimum coordinates. If false, the walker can go outside these bounds. In that
	 *                    case the boundaries are only used to calculate the step size.
	 */
	bool init(Function *pProbFunction, const double *pMinCoords, const double *pMaxCoords, const double *pStartCoords, 
	          bool keepInside = true);

	/** Set this position as the current position of the walker. */
	void setNewStartCoordinates(double *pStartCoords);

	void walk(size_t numSteps);
	const double *getBestCoordinates() const							{ return &(m_bestCoords[0]); }
protected:
	/** By overriding this function, you can change the way the walker steps to a new coordinate 
	 *  (be careful if you change this, without obeying certain rules, the underlying probability
	 *  distribution will not be sampled correctly). */
	virtual void stepToNewCoords(const double *pCurCoords, double *pNewCoords);
private:
	std::vector<double> m_minCoords, m_maxCoords;
	std::vector<double> m_coords, m_tryCoords;
	std::vector<double> m_stepScale;
	std::vector<double> m_bestCoords;
	bool m_keepInside;
};

} // end namespace   

#endif // MEANWALKER_METROPOLISWALKER_H
